/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/

using System;
using System.Timers;
using System.Drawing;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using NeuralNetwork;

namespace RobotController
{
	/// <summary>
	/// Summary description for NeuralNetInterface.
	/// </summary>
	public class NeuralNetInterface
	{
		private System.Timers.Timer checkNN;
		private WebcamInterface cam;
		private NeuralNetwork.NeuralNetwork myNet;
		private Robot myRobot;

		private int consecutiveDetections;

		private const float NN_THREASHOLD = 10;

		public NeuralNetInterface(Robot rob, WebcamInterface wi)
		{
			cam = wi;
			checkNN = new System.Timers.Timer();
			checkNN.Elapsed += new System.Timers.ElapsedEventHandler(check);
			checkNN.Interval = 100;
			myRobot = rob;
			consecutiveDetections=0;

			loadNN();
		}

		/// <summary>
		/// Starts the neural nework checking the video stream 
		/// </summary>
		public void Start()
		{
			checkNN.Start();
		}

		/// <summary>
		/// Stops the neural network interface's timer
		/// </summary>
		public void Stop()
		{
			checkNN.Stop();
		}

		/// <summary>
		/// Called every interval of checkNN, checks the current fram of cam through the neural net
		/// </summary>
		private void check(object source, ElapsedEventArgs e) 
		{
			if(myRobot.Status != StatusType.anomoly_detected && myRobot.Status != StatusType.user_control && myRobot.Status != StatusType.idle)
			{
				float[] img = cam.getNeuralNetImage();
				float[] result = myNet.Output(img);
				double k = (double)result[0];
				k = k * 1E32;  //k is very small, multiply it up to an acceptable level

				if (k >= NN_THREASHOLD)  //if the value read from the neuralnet is above the threashold, detection
				{	
					consecutiveDetections++;  //increment the detection buffer
					if(consecutiveDetections == 5) //number of consecutive detections needed before alert is sent
					{
						myRobot.Status = StatusType.anomoly_detected;  //change robot status
						myRobot.stop(); //stop
						consecutiveDetections=0;
					}
				}
				else
				{
					consecutiveDetections = 0;  //reset consecutivedetections if nothing was detected
				}
			}

		}

		/// <summary>
		/// Loads the neural net from "default.nn" in the working directory
		/// </summary>
		private void loadNN()
		{
			//myNet = new NeuralNetwork.NeuralNetwork(0,int[] i = new int[0]);
			FileStream sr = File.Open("default.nn", FileMode.Open); 
			IFormatter binFmt = new BinaryFormatter();
			myNet = (NeuralNetwork.NeuralNetwork)binFmt.Deserialize(sr);
							
			sr.Close();

		}
	}
}
