/***********************************
 *        Timothy Hannafin         *
 *        hannafit@wpi.edu         *
 * Worcester Polytechnic Institute *
 *               2006              *
 * *********************************/

using System;
using System.Timers;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Data;
using System.Text;
using System.IO;
using System.Drawing;


namespace RobotController
{
	/// <summary>
	/// A server object contains the various net services required to operate the robot 
	/// remotly including the command, feedback, and image server
	/// </summary>
	public class Server
	{
		private Thread commandThread;
		private Thread feedbackThread;
		private Thread imageThread;
		private Robot myRobot;
		private RobotController.Controller form1;
		float speed = 0;

		private CommandServer cmdServer;
		private FeedbackServer fdbServer;
		private ImageServer imgServer;
		private NeuralNetInterface nNet;

		private WebcamInterface camInterface;

		public Server(Robot aRobot, RobotController.Controller aform)
		{
			myRobot = aRobot;
			form1 = aform;
			speed = form1.speed;
			
		}

		/// <summary>
		/// Start the robot server
		/// </summary>

		public void Start()
		{
			
			startCamera();

			/* TO DISABLE NEURAL NETWORK COMMENT THESE TWO LINES */
			nNet = new NeuralNetInterface(myRobot, camInterface);
			nNet.Start();

			commandThread = new Thread(new ThreadStart(startCommandServer));
			commandThread.Start();

			imageThread = new Thread(new ThreadStart(StartImageServer));
			imageThread.Start();

			feedbackThread = new Thread(new ThreadStart(StartFeedbackServer));
			feedbackThread.Start();


		}

		/// <summary>
		/// Start the webcam interface
		/// </summary>
		private void startCamera()
		{
			camInterface = new WebcamInterface(form1);
			camInterface.start();
		}

		/// <summary>
		/// Stop the server
		/// </summary>
		public void stop()
		{
			//this function does not work properly, threads will continue to run for a period of time
			//and if the program is run outside of the VS.net environment
			//they may have to be killed using windows task manager
			if(commandThread.IsAlive)
			{
				cmdServer.Stop();
				commandThread.Suspend();
				imageThread.Suspend();
				commandThread.Abort();
				imageThread.Abort();
				nNet.Stop();
			}
		}

		/// <summary>
		/// Add a message to the stus window on the main form
		/// </summary>
		/// <param name="message"></param>
		private void addStatus(string message)
		{
			form1.txtStatus.AppendText(message + System.Environment.NewLine);

		}

		/// <summary>
		/// Start the image server
		/// </summary>
		private void StartImageServer()
		{
			String hostName = System.Net.Dns.GetHostName();		
			IPHostEntry ipEntry = System.Net.Dns.GetHostByName(hostName);
			IPAddress[] addr = ipEntry.AddressList;
			
			imgServer = new ImageServer(addr[0].ToString(), 9986, myRobot, form1, camInterface);
			imgServer.Start();
		}

		/// <summary>
		/// Start the feedback server
		/// </summary>
		private void StartFeedbackServer()
		{
				String hostName = System.Net.Dns.GetHostName();		
				IPHostEntry ipEntry = System.Net.Dns.GetHostByName(hostName);
				IPAddress[] addr = ipEntry.AddressList;
			
				fdbServer = new FeedbackServer(addr[0].ToString(), 9987, myRobot);
				fdbServer.Start();
		}

		/// <summary>
		/// Start the command server
		/// </summary>
		private void startCommandServer()
		{
				String hostName = System.Net.Dns.GetHostName();
			
				IPHostEntry ipEntry = System.Net.Dns.GetHostByName(hostName);
				IPAddress[] addr = ipEntry.AddressList;

				cmdServer = new CommandServer(addr[0].ToString(), 9988, myRobot, form1);
				cmdServer.Start();
		}

		public void setSpeed(float i)
		{
			speed = i;	
		}
	}
}
